"""
Auth hooks for Slate Bootstrap Theme for Alliance Auth
"""

# Alliance Auth
from allianceauth import hooks
from allianceauth.theme.hooks import ThemeHook

# AA Theme Slate
from aa_theme_slate.helper.static_files import get_theme_hook_static


class AaSlateThemeHook(ThemeHook):
    """
    Slate Bootstrap Theme for Alliance Auth
    https://github.com/ppfeufer/aa-theme-slate
    """

    def __init__(self):
        """
        Init
        """

        css_static_files = [
            get_theme_hook_static(
                static_file="theme/aav4/libs/bootswatch/v5.3.3/slate/css/bootstrap.min.css"
            ),
            get_theme_hook_static(
                static_file="theme/aav4/css/community-apps.min.css",
                with_version=True,
            ),
        ]

        js_static_files = [
            get_theme_hook_static(
                static_file="theme/aav4/libs/popper/v2.11.8/popper.min.js"
            ),
            get_theme_hook_static(
                static_file="theme/aav4/libs/bootswatch/v5.3.3/slate/javascript/bootstrap.min.js"
            ),
        ]

        ThemeHook.__init__(
            self=self,
            name="Slate",
            description="Slate Bootstrap Theme for Alliance Auth",
            html_tags={"data-theme": "slate"},
            css=css_static_files,
            js=js_static_files,
            header_padding="3.6em",
        )


@hooks.register("theme_hook")
def register_aa_slate_hook():
    """
    Registers the Slate Bootstrap Theme for Alliance Auth hook

    :return:
    :rtype:
    """

    return AaSlateThemeHook()



"""
Helper functions for static integrity calculations
"""

# Standard Library
import os
from pathlib import Path
from urllib.parse import urljoin

# Third Party
from sri import Algorithm, calculate_integrity

# Django
from django.conf import settings

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA Theme Slate
from aa_theme_slate import __title__, __version__
from aa_theme_slate.constants import SLATE_STATIC_DIR

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


def calculate_integrity_hash(relative_file_path: str) -> str:
    """
    Calculates the integrity hash for a given static file

    :param self:
    :type self:
    :param relative_file_path: The file path relative to the `aa_theme_slate/static/aa_theme_slate` folder
    :type relative_file_path: str
    :return: The integrity hash
    :rtype: str
    """

    file_path = os.path.join(SLATE_STATIC_DIR, relative_file_path)
    integrity_hash = calculate_integrity(
        path=Path(file_path), algorithm=Algorithm.SHA512
    )

    return integrity_hash


def get_theme_hook_static(
    static_file: str, script_type: str = None, with_version: bool = False
) -> dict:
    """
    Get the static file details for a theme hook

    :param static_file: The file path relative to the `aa_theme_slate/static/aa_theme_slate` folder
    :type static_file: str
    :param script_type: The script type
    :type script_type: str
    :param with_version: Includes the version number in the URL
    :type with_version: bool
    :return: The static file details
    :rtype: dict
    """

    return_value = {
        "url": urljoin(
            base=settings.STATIC_URL,
            url=f"aa_theme_slate/{static_file}",
        ),
    }

    # Calculate integrity hash if not in debug mode
    if not settings.DEBUG:
        integrity = calculate_integrity_hash(relative_file_path=static_file)
        return_value["integrity"] = integrity

    # Add the script type if provided
    if script_type:
        return_value["js_type"] = script_type

    # Add the version number if requested
    if with_version:
        return_value["url"] += f"?v={__version__}"

    return return_value
